@@ -1,12 +1,14 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: April 21, 2016
-# URL: https://humanfriendly.readthedocs.org
+# Last Change: July 16, 2017
+# URL: https://humanfriendly.readthedocs.io
 
 """The main module of the `humanfriendly` package."""
 
 # Standard library modules.
+import collections
+import decimal
 import multiprocessing
 import numbers
 import os
@@ -34,10 +36,10 @@ from humanfriendly.text import (  # NOQA
 from humanfriendly.prompts import prompt_for_choice  # NOQA
 
 # Compatibility with Python 2 and 3.
-from humanfriendly.compat import is_string
+from humanfriendly.compat import is_string, monotonic
 
 # Semi-standard module versioning.
-__version__ = '1.44.7'
+__version__ = '4.4'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -50,13 +52,17 @@ erase_line_code = '\r\x1b[K'
 hide_cursor_code = '\x1b[?25l'
 show_cursor_code = '\x1b[?25h'
 
-# Common disk size units, used for formatting and parsing.
-disk_size_units = (dict(prefix='b', divider=1, singular='byte', plural='bytes'),
-                   dict(prefix='k', divider=1024**1, singular='KB', plural='KB'),
-                   dict(prefix='m', divider=1024**2, singular='MB', plural='MB'),
-                   dict(prefix='g', divider=1024**3, singular='GB', plural='GB'),
-                   dict(prefix='t', divider=1024**4, singular='TB', plural='TB'),
-                   dict(prefix='p', divider=1024**5, singular='PB', plural='PB'))
+SizeUnit = collections.namedtuple('SizeUnit', 'divider, symbol, name')
+CombinedUnit = collections.namedtuple('CombinedUnit', 'decimal, binary')
+
+# Common disk size units in binary (base-2) and decimal (base-10) multiples.
+disk_size_units = (
+    CombinedUnit(SizeUnit(1000**1, 'KB', 'kilobyte'), SizeUnit(1024**1, 'KiB', 'kibibyte')),
+    CombinedUnit(SizeUnit(1000**2, 'MB', 'megabyte'), SizeUnit(1024**2, 'MiB', 'mebibyte')),
+    CombinedUnit(SizeUnit(1000**3, 'GB', 'gigabyte'), SizeUnit(1024**3, 'GiB', 'gibibyte')),
+    CombinedUnit(SizeUnit(1000**4, 'TB', 'terabyte'), SizeUnit(1024**4, 'TiB', 'tebibyte')),
+    CombinedUnit(SizeUnit(1000**5, 'PB', 'petabyte'), SizeUnit(1024**5, 'PiB', 'pebibyte')),
+)
 
 # Common length size units, used for formatting and parsing.
 length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'),
@@ -66,13 +72,13 @@ length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'
                      dict(prefix='km', divider=1000, singular='km', plural='km'))
 
 # Common time units, used for formatting of time spans.
-time_units = (dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviation='ms'),
-              dict(divider=1, singular='second', plural='seconds', abbreviation='s'),
-              dict(divider=60, singular='minute', plural='minutes', abbreviation='m'),
-              dict(divider=60*60, singular='hour', plural='hours', abbreviation='h'),
-              dict(divider=60*60*24, singular='day', plural='days', abbreviation='d'),
-              dict(divider=60*60*24*7, singular='week', plural='weeks', abbreviation='w'),
-              dict(divider=60*60*24*7*52, singular='year', plural='years', abbreviation='y'))
+time_units = (dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviations=['ms']),
+              dict(divider=1, singular='second', plural='seconds', abbreviations=['s', 'sec', 'secs']),
+              dict(divider=60, singular='minute', plural='minutes', abbreviations=['m', 'min', 'mins']),
+              dict(divider=60 * 60, singular='hour', plural='hours', abbreviations=['h']),
+              dict(divider=60 * 60 * 24, singular='day', plural='days', abbreviations=['d']),
+              dict(divider=60 * 60 * 24 * 7, singular='week', plural='weeks', abbreviations=['w']),
+              dict(divider=60 * 60 * 24 * 7 * 52, singular='year', plural='years', abbreviations=['y']))
 
 
 def coerce_boolean(value):
@@ -81,13 +87,13 @@ def coerce_boolean(value):
 
     :param value: Any Python value. If the value is a string:
 
-                  - The strings '1', 'yes', 'true' and 'on' are coerced to ``True``.
-                  - The strings '0', 'no', 'false' and 'off' are coerced to ``False``.
+                  - The strings '1', 'yes', 'true' and 'on' are coerced to :data:`True`.
+                  - The strings '0', 'no', 'false' and 'off' are coerced to :data:`False`.
                   - Other strings raise an exception.
 
-                  Other Python values are coerced using :py:func:`bool()`.
+                  Other Python values are coerced using :func:`bool()`.
     :returns: A proper boolean value.
-    :raises: :py:exc:`exceptions.ValueError` when the value is a string but
+    :raises: :exc:`exceptions.ValueError` when the value is a string but
              cannot be coerced with certainty.
     """
     if is_string(value):
@@ -103,18 +109,19 @@ def coerce_boolean(value):
         return bool(value)
 
 
-def format_size(num_bytes, keep_width=False):
+def format_size(num_bytes, keep_width=False, binary=False):
     """
     Format a byte count as a human readable file size.
 
     :param num_bytes: The size to format in bytes (an integer).
-    :param keep_width: ``True`` if trailing zeros should not be stripped,
-                       ``False`` if they can be stripped.
+    :param keep_width: :data:`True` if trailing zeros should not be stripped,
+                       :data:`False` if they can be stripped.
+    :param binary: :data:`True` to use binary multiples of bytes (base-2),
+                   :data:`False` to use decimal multiples of bytes (base-10).
     :returns: The corresponding human readable file size (a string).
 
-    This function supports ranges from kilobytes to terabytes.
-
-    Some examples:
+    This function knows how to format sizes in bytes, kilobytes, megabytes,
+    gigabytes, terabytes and petabytes. Some examples:
 
     >>> from humanfriendly import format_size
     >>> format_size(0)
@@ -123,50 +130,79 @@ def format_size(num_bytes, keep_width=False):
     '1 byte'
     >>> format_size(5)
     '5 bytes'
-    >>> format_size(1024 ** 2)
-    '1 MB'
-    >>> format_size(1024 ** 3 * 4)
+    > format_size(1000)
+    '1 KB'
+    > format_size(1024, binary=True)
+    '1 KiB'
+    >>> format_size(1000 ** 3 * 4)
     '4 GB'
     """
     for unit in reversed(disk_size_units):
-        if num_bytes >= unit['divider']:
-            number = round_number(float(num_bytes) / unit['divider'], keep_width=keep_width)
-            return pluralize(number, unit['singular'], unit['plural'])
+        if num_bytes >= unit.binary.divider and binary:
+            number = round_number(float(num_bytes) / unit.binary.divider, keep_width=keep_width)
+            return pluralize(number, unit.binary.symbol, unit.binary.symbol)
+        elif num_bytes >= unit.decimal.divider and not binary:
+            number = round_number(float(num_bytes) / unit.decimal.divider, keep_width=keep_width)
+            return pluralize(number, unit.decimal.symbol, unit.decimal.symbol)
     return pluralize(num_bytes, 'byte')
 
 
-def parse_size(size):
+def parse_size(size, binary=False):
     """
     Parse a human readable data size and return the number of bytes.
 
     :param size: The human readable file size to parse (a string).
+    :param binary: :data:`True` to use binary multiples of bytes (base-2) for
+                   ambiguous unit symbols and names, :data:`False` to use
+                   decimal multiples of bytes (base-10).
     :returns: The corresponding size in bytes (an integer).
     :raises: :exc:`InvalidSize` when the input can't be parsed.
 
-    Some examples:
+    This function knows how to parse sizes in bytes, kilobytes, megabytes,
+    gigabytes, terabytes and petabytes. Some examples:
 
     >>> from humanfriendly import parse_size
     >>> parse_size('42')
     42
+    >>> parse_size('13b')
+    13
+    >>> parse_size('5 bytes')
+    5
     >>> parse_size('1 KB')
+    1000
+    >>> parse_size('1 kilobyte')
+    1000
+    >>> parse_size('1 KiB')
+    1024
+    >>> parse_size('1 KB', binary=True)
     1024
-    >>> parse_size('5 kilobyte')
-    5120
     >>> parse_size('1.5 GB')
+    1500000000
+    >>> parse_size('1.5 GB', binary=True)
     1610612736
     """
     tokens = tokenize(size)
     if tokens and isinstance(tokens[0], numbers.Number):
-        # If the input contains only a number, it's assumed to be the number of bytes.
-        if len(tokens) == 1:
+        # Get the normalized unit (if any) from the tokenized input.
+        normalized_unit = tokens[1].lower() if len(tokens) == 2 and is_string(tokens[1]) else ''
+        # If the input contains only a number, it's assumed to be the number of
+        # bytes. The second token can also explicitly reference the unit bytes.
+        if len(tokens) == 1 or normalized_unit.startswith('b'):
             return int(tokens[0])
-        # Otherwise we expect to find two tokens: A number and a unit.
-        if len(tokens) == 2 and is_string(tokens[1]):
-            normalized_unit = tokens[1].lower()
-            # Try to match the first letter of the unit.
+        # Otherwise we expect two tokens: A number and a unit.
+        if normalized_unit:
             for unit in disk_size_units:
-                if normalized_unit.startswith(unit['prefix']):
-                    return int(tokens[0] * unit['divider'])
+                # First we check for unambiguous symbols (KiB, MiB, GiB, etc)
+                # and names (kibibyte, mebibyte, gibibyte, etc) because their
+                # handling is always the same.
+                if normalized_unit in (unit.binary.symbol.lower(), unit.binary.name.lower()):
+                    return int(tokens[0] * unit.binary.divider)
+                # Now we will deal with ambiguous prefixes (K, M, G, etc),
+                # symbols (KB, MB, GB, etc) and names (kilobyte, megabyte,
+                # gigabyte, etc) according to the caller's preference.
+                if (normalized_unit in (unit.decimal.symbol.lower(), unit.decimal.name.lower()) or
+                        normalized_unit.startswith(unit.decimal.symbol[0].lower())):
+                    return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
     # We failed to parse the size specification.
     msg = "Failed to parse size! (input %r was tokenized as %r)"
     raise InvalidSize(msg % (size, tokens))
@@ -177,8 +213,8 @@ def format_length(num_metres, keep_width=False):
     Format a metre count as a human readable length.
 
     :param num_metres: The length to format in metres (float / integer).
-    :param keep_width: ``True`` if trailing zeros should not be stripped,
-                       ``False`` if they can be stripped.
+    :param keep_width: :data:`True` if trailing zeros should not be stripped,
+                       :data:`False` if they can be stripped.
     :returns: The corresponding human readable length (a string).
 
     This function supports ranges from nanometres to kilometres.
@@ -283,8 +319,8 @@ def round_number(count, keep_width=False):
     Round a floating point number to two decimal places in a human friendly format.
 
     :param count: The number to format.
-    :param keep_width: ``True`` if trailing zeros should not be stripped,
-                       ``False`` if they can be stripped.
+    :param keep_width: :data:`True` if trailing zeros should not be stripped,
+                       :data:`False` if they can be stripped.
     :returns: The formatted number as a string. If no decimal places are
               required to represent the number, they will be omitted.
 
@@ -309,7 +345,7 @@ def round_number(count, keep_width=False):
     return text
 
 
-def format_timespan(num_seconds, detailed=False):
+def format_timespan(num_seconds, detailed=False, max_units=3):
     """
     Format a timespan in seconds as a human readable string.
 
@@ -317,6 +353,8 @@ def format_timespan(num_seconds, detailed=False):
     :param detailed: If :data:`True` milliseconds are represented separately
                      instead of being represented as fractional seconds
                      (defaults to :data:`False`).
+    :param max_units: The maximum number of units to show in the formatted time
+                      span (an integer, defaults to three).
     :returns: The formatted timespan as a string.
 
     Some examples:
@@ -341,18 +379,30 @@ def format_timespan(num_seconds, detailed=False):
     else:
         # Slow path.
         result = []
-        for unit in reversed(time_units):
-            if num_seconds >= unit['divider']:
-                count = int(num_seconds / unit['divider'])
-                num_seconds %= unit['divider']
+        num_seconds = decimal.Decimal(str(num_seconds))
+        relevant_units = list(reversed(time_units[0 if detailed else 1:]))
+        for unit in relevant_units:
+            # Extract the unit count from the remaining time.
+            divider = decimal.Decimal(str(unit['divider']))
+            count = num_seconds / divider
+            num_seconds %= divider
+            # Round the unit count appropriately.
+            if unit != relevant_units[-1]:
+                # Integer rounding for all but the smallest unit.
+                count = int(count)
+            else:
+                # Floating point rounding for the smallest unit.
+                count = round_number(count)
+            # Only include relevant units in the result.
+            if count not in (0, '0'):
                 result.append(pluralize(count, unit['singular'], unit['plural']))
         if len(result) == 1:
             # A single count/unit combination.
             return result[0]
         else:
             if not detailed:
-                # Remove insignificant data from the formatted timespan.
-                result = result[:3]
+                # Remove `insignificant' data from the formatted timespan.
+                result = result[:max_units]
             # Format the timespan in a readable way.
             return concatenate(result)
 
@@ -372,6 +422,16 @@ def parse_timespan(timespan):
     minimal amount of typing. It's very useful to accept easy to write time
     spans as e.g. command line arguments to programs.
 
+    The time units (and abbreviations) supported by this function are:
+
+    - ms, millisecond, milliseconds
+    - s, sec, secs, second, seconds
+    - m, min, mins, minute, minutes
+    - h, hour, hours
+    - d, day, days
+    - w, week, weeks
+    - y, year, years
+
     Some examples:
 
     >>> from humanfriendly import parse_timespan
@@ -395,7 +455,9 @@ def parse_timespan(timespan):
         if len(tokens) == 2 and is_string(tokens[1]):
             normalized_unit = tokens[1].lower()
             for unit in time_units:
-                if normalized_unit in (unit['singular'], unit['plural'], unit['abbreviation']):
+                if (normalized_unit == unit['singular'] or
+                        normalized_unit == unit['plural'] or
+                        normalized_unit in unit['abbreviations']):
                     return float(tokens[0]) * unit['divider']
     # We failed to parse the timespan specification.
     msg = "Failed to parse timespan! (input %r was tokenized as %r)"
@@ -438,15 +500,15 @@ def parse_date(datestring):
     >>> mktime(parse_date('2013-06-17 02:47:42') + (-1, -1, -1))
     1371430062.0
 
-    And here's how you convert it to a :py:class:`datetime.datetime` object:
+    And here's how you convert it to a :class:`datetime.datetime` object:
 
     >>> from humanfriendly import parse_date
     >>> from datetime import datetime
     >>> datetime(*parse_date('2013-06-17 02:47:42'))
     datetime.datetime(2013, 6, 17, 2, 47, 42)
 
-    Here's an example that combines :py:func:`format_timespan()` and
-    :py:func:`parse_date()` to calculate a human friendly timespan since a
+    Here's an example that combines :func:`format_timespan()` and
+    :func:`parse_date()` to calculate a human friendly timespan since a
     given date:
 
     >>> from humanfriendly import format_timespan, parse_date
@@ -456,11 +518,11 @@ def parse_date(datestring):
     >>> print(format_timespan(seconds_since_then))
     1 year, 43 weeks and 1 day
 
-    .. _dateutil: https://dateutil.readthedocs.org/en/latest/parser.html
+    .. _dateutil: https://dateutil.readthedocs.io/en/latest/parser.html
     .. _Unix time: http://en.wikipedia.org/wiki/Unix_time
     """
     try:
-        tokens = list(map(str.strip, datestring.split()))
+        tokens = [t.strip() for t in datestring.split()]
         if len(tokens) >= 2:
             date_parts = list(map(int, tokens[0].split('-'))) + [1, 1]
             time_parts = list(map(int, tokens[1].split(':'))) + [0, 0, 0]
@@ -509,9 +571,9 @@ def parse_path(pathname):
     """
     Convert a human friendly pathname to an absolute pathname.
 
-    Expands leading tildes using :py:func:`os.path.expanduser()` and
-    environment variables using :py:func:`os.path.expandvars()` and makes the
-    resulting pathname absolute using :py:func:`os.path.abspath()`.
+    Expands leading tildes using :func:`os.path.expanduser()` and
+    environment variables using :func:`os.path.expandvars()` and makes the
+    resulting pathname absolute using :func:`os.path.abspath()`.
 
     :param pathname: A human friendly pathname (a string).
     :returns: An absolute pathname (a string).
@@ -527,34 +589,51 @@ class Timer(object):
 
     def __init__(self, start_time=None, resumable=False):
         """
-        Remember the time when the :py:class:`Timer` was created.
+        Remember the time when the :class:`Timer` was created.
 
         :param start_time: The start time (a float, defaults to the current time).
-        :param resumable: Create a resumable timer (defaults to ``False``).
+        :param resumable: Create a resumable timer (defaults to :data:`False`).
+
+        When `start_time` is given :class:`Timer` uses :func:`time.time()` as a
+        clock source, otherwise it uses :func:`humanfriendly.compat.monotonic()`.
         """
-        self.resumable = resumable
-        if self.resumable:
+        if resumable:
+            self.monotonic = True
+            self.resumable = True
             self.start_time = 0.0
             self.total_time = 0.0
+        elif start_time:
+            self.monotonic = False
+            self.resumable = False
+            self.start_time = start_time
         else:
-            self.start_time = start_time or time.time()
+            self.monotonic = True
+            self.resumable = False
+            self.start_time = monotonic()
 
     def __enter__(self):
         """
         Start or resume counting elapsed time.
+
+        :returns: The :class:`Timer` object.
+        :raises: :exc:`~exceptions.ValueError` when the timer isn't resumable.
         """
         if not self.resumable:
             raise ValueError("Timer is not resumable!")
-        self.start_time = time.time()
+        self.start_time = monotonic()
+        return self
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
         """
         Stop counting elapsed time.
+
+        :raises: :exc:`~exceptions.ValueError` when the timer isn't resumable.
         """
         if not self.resumable:
             raise ValueError("Timer is not resumable!")
-        self.total_time += time.time() - self.start_time
-        self.start_time = 0
+        if self.start_time:
+            self.total_time += monotonic() - self.start_time
+            self.start_time = 0.0
 
     @property
     def elapsed_time(self):
@@ -565,7 +644,8 @@ class Timer(object):
         if self.resumable:
             elapsed_time += self.total_time
         if self.start_time:
-            elapsed_time += time.time() - self.start_time
+            current_time = monotonic() if self.monotonic else time.time()
+            elapsed_time += current_time - self.start_time
         return elapsed_time
 
     @property
@@ -659,8 +739,8 @@ class Spinner(object):
                 progress += random.random() * 5
 
     If you want to provide user feedback during a long running operation but
-    it's not practical to periodically call the :py:func:`~Spinner.step()`
-    method consider using :py:class:`AutomaticSpinner` instead.
+    it's not practical to periodically call the :func:`~Spinner.step()`
+    method consider using :class:`AutomaticSpinner` instead.
 
     As you may already have noticed in the examples above, :class:`Spinner`
     objects can be used as context managers to automatically call
@@ -673,17 +753,17 @@ class Spinner(object):
         """
         Initialize a spinner.
 
-        :param label: The label for the spinner (a string, defaults to ``None``).
+        :param label: The label for the spinner (a string, defaults to :data:`None`).
         :param total: The expected number of steps (an integer).
         :param stream: The output stream to show the spinner on (defaults to
-                       :py:data:`sys.stderr`).
-        :param interactive: If this is ``False`` then the spinner doesn't write
+                       :data:`sys.stderr`).
+        :param interactive: If this is :data:`False` then the spinner doesn't write
                             to the output stream at all. It defaults to the
                             return value of ``stream.isatty()``.
-        :param timer: A :py:class:`Timer` object (optional). If this is given
+        :param timer: A :class:`Timer` object (optional). If this is given
                       the spinner will show the elapsed time according to the
                       timer.
-        :param hide_cursor: If ``True`` (the default) the text cursor is hidden
+        :param hide_cursor: If :data:`True` (the default) the text cursor is hidden
                             as long as the spinner is active.
         """
         self.label = label
@@ -737,7 +817,7 @@ class Spinner(object):
                 if not label:
                     raise Exception("No label set for spinner!")
                 elif self.total and progress:
-                    label = "%s: %.2f%%" % (label, progress/(self.total/100.0))
+                    label = "%s: %.2f%%" % (label, progress / (self.total / 100.0))
                 elif self.timer and self.timer.elapsed_time > 2:
                     label = "%s (%s)" % (label, self.timer.rounded)
                 self.stream.write("%s %s %s ..\r" % (erase_line_code, state, label))
@@ -772,11 +852,15 @@ class Spinner(object):
             self.stream.write(erase_line_code)
 
     def __enter__(self):
-        """Enable the use of spinners as context managers."""
+        """
+        Enable the use of spinners as context managers.
+
+        :returns: The :class:`Spinner` object.
+        """
         return self
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
-        """Enable the use of spinners as context managers."""
+        """Clear the spinner when leaving the context."""
         self.clear()
 
 
@@ -785,20 +869,20 @@ class AutomaticSpinner(object):
     """
     Show a spinner on the terminal that automatically starts animating.
 
-    This class shows a spinner on the terminal (just like :py:class:`Spinner`
+    This class shows a spinner on the terminal (just like :class:`Spinner`
     does) that automatically starts animating. This class should be used as a
-    context manager using the :py:keyword:`with` statement. The animation
+    context manager using the :keyword:`with` statement. The animation
     continues for as long as the context is active.
 
-    :py:class:`AutomaticSpinner` provides an alternative to :py:class:`Spinner`
+    :class:`AutomaticSpinner` provides an alternative to :class:`Spinner`
     for situations where it is not practical for the caller to periodically
-    call :py:func:`~Spinner.step()` to advance the animation, e.g. because
+    call :func:`~Spinner.step()` to advance the animation, e.g. because
     you're performing a blocking call and don't fancy implementing threading or
     subprocess handling just to provide some user feedback.
 
-    This works using the :py:mod:`multiprocessing` module by spawning a
+    This works using the :mod:`multiprocessing` module by spawning a
     subprocess to render the spinner while the main process is busy doing
-    something more useful. By using the :py:keyword:`with` statement you're
+    something more useful. By using the :keyword:`with` statement you're
     guaranteed that the subprocess is properly terminated at the appropriate
     time.
     """
@@ -808,7 +892,7 @@ class AutomaticSpinner(object):
         Initialize an automatic spinner.
 
         :param label: The label for the spinner (a string).
-        :param show_time: If this is ``True`` (the default) then the spinner
+        :param show_time: If this is :data:`True` (the default) then the spinner
                           shows elapsed time.
         """
         self.label = label
